<?php
//Define some constants for the various stages
define('BJ_LAST_CALL', 1);
define('BJ_END', 3);
define('BJ_BETTING_R1', 4);
define('BJ_BETTING_R2', 5);
define('BJ_HIT_STAND', 6);

class BJPlayer {
	var $Money;
	var $Cards;
	var $Nick;
	var $Bet;
	function BJPlayer($Nick){
		$this->Money=100;
		$this->Cards=array();
		$this->Nick=$Nick;
		$this->Bet=0;
	}
	
	function getNick(){
		return $this->Nick;
	}

	function clearBet(){
		$this->Bet=0;
	}

	function placeBet($amount){
		$this->Bet+=$amount;
	}
	
	function getBet(){
		return $this->Bet;
	}

	function deduct($amount){
		$this->Money-=$amount;
	}

	function deposit($amount){
		$this->Money+=$amount;
	}

	function getMoney(){
		return $this->Money;
	}

	function give($Card){
		$this->Cards[]=$Card;
	}

	function clearCards(){
		$this->Cards=array();
	}

	function getHandText(){
		$Total=count($this->Cards);
		$out='';
		for ($i=0; $i<$Total; $i++){
			$card=$this->Cards[$i];
			$out.=Deck::getCardText($card).'  ';
		}
		$out.=' -- Total: '.$this->getHandTotal();
		return $out;
	}

	function getHandTotal(){
		return getHandTotal($this->Cards);
	}

}

//Classes that interface with the db
class BJDealer extends BJPlayer {
	var $Money;
	var $Deck;
	var $Players;
	var $Channel;
	var $CurrentPlayer;
	var $BustList;
	var $Stage;

	function BJDealer($Chan){
		$this->Money=1000;
		$this->Deck=new Deck();
		$this->Players=array();
		$this->Channel=$Chan;
		$this->CurrentPlayer=0;
		$this->BustList=array();
		$this->Stage=BJ_END;
	}

	function getStage(){
		return $this->Stage;
	}
	
	function setStage($stage){
		$this->Stage=$stage;
	}

	function isPlaying(){
		return $this->Stage != BJ_END && $this->Stage != BJ_LAST_CALL;
	}

	function setCurrentPlayer($i){
		$this->CurrentPlayer=$i;
	}

	function getCurrentplayer(){
		return $this->CurrentPlayer;
	}

	function addPlayer($Nick){
		if (in_array($Nick, $this->Players)){
			return 2;
		}
		else if (count($this->Players) >= 10){
			return 0;
		}
		$this->Players[]=$Nick;
		return 1;
	}

	function removePlayer($Nick){
		if (in_array($Nick, $this->Players)){
			//print_r($this->Players);
			$key=array_search($Nick, $this->Players);
			array_splice($this->Players, $key, 1);
			$key=array_search($Nick, $this->BustList);
			if ($key !== false){
				array_splice($this->BustList, $key, 1);
			}
			//print_r($this->Players);
			$this->CurrentPlayer--;
			return true;
		}
		else {
			return false;
		}
	}

	function clearPlayers(){
		$this->BustList=array();
		$this->Players=array();
		return count($this->Players)==0 && count($this->BustList)==0;
	}

	function getTotalPlayers(){
		return count($this->Players);
	}

	function nextPlayer(){
		$t=count($this->Players);
		$cp=$this->CurrentPlayer;
		$start=$cp;
		$cp++;
		if ($cp >= $t){
			$cp=0;
			while ($this->playerIsBust($this->Players[$cp])){
				$cp++;
				if ($cp>=$t){
					//Made a complete loop
					$this->CurrentPlayer=0;
					return false;
				}
			}
			$this->CurrentPlayer=$cp;
			return false;
		}
		while ($this->playerIsBust($this->Players[$cp])){
			$cp++;
			if ($cp>=$t){
				//Hit the end of the list, reset
				$this->CurrentPlayer=0;
				return false;
			}
			if ($cp == $start){
				//Made complete loop.  All busted.
				$this->CurrentPlayer=0;
				return false;
			}
		}
		$this->CurrentPlayer=$cp;
		return true;
	}

	function playerBusted($Player){
		$this->BustList[]=$Player;
	}

	function playerIsBust($Player){
		return in_array($Player, $this->BustList);
	}

	function allBust(){
		return count($this->BustList) >= count($this->Players);
	}

	function getPlayer($i){
		$res=db_query("SELECT Data FROM mod_blackjack_players WHERE Nick='".addslashes($this->Players[$i])."' AND Channel='".addslashes($this->Channel)."'");
		if (!$res || db_num_rows($res)==0){
			return false;
		}
		return unserialize(db_result($res, 0, 0));
	}

	function draw(){
		return $this->Deck->getRandomCard();
	}

	function reshuffle(){
		$this->Deck->rebuild();
	}
}


class Deck {
	var $Cards;
	function Deck(){
		$this->Cards=$this->buildDeck();
	}

	function rebuild(){
		$this->Cards=$this->buildDeck();
	}

	function buildDeck(){
		return array(	'HA', 'H2', 'H3', 'H4', 'H5', 'H6', 'H7', 'H8', 'H9', 'HJ', 'HQ', 'HK', /*Hearts*/
				'DA', 'D2', 'D3', 'D4', 'D5', 'D6', 'D7', 'D8', 'D9', 'DJ', 'DQ', 'DK', /*Diamonds*/
				'SA', 'S2', 'S3', 'S4', 'S5', 'S6', 'S7', 'S8', 'S9', 'SJ', 'SQ', 'SK', /*Spades*/
				'CA', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9', 'CJ', 'CQ', 'CK'  /*Clubs*/
			);
	}
	function getRandomCard(){
		$ran=array_rand($this->Cards);
		$card=$this->Cards[$ran];
		$this->removeCard($card);
		return $card;
	}

	function removeCard($Code){
		$key=array_search($Code, $this->Cards);
		if ($key===false){
			return false;
		}
		unset($this->Cards[$key]);
		return true;
	}

	function cardRemoved($Code){
		return in_array($Code, $this->Cards);
	}
			
	function getCardText($Code){
		$text="";
		switch (strtoupper($Code{0})){
			case 'H':
				$text='(Heart)';
				break;
			case 'D':
				$text='(Diamond)';
				break;
			case 'S':
				$text='(Spade)';
				break;
			case 'C':
				$text='(Club)';
				break;
			default:
				return false;
		}
		switch ($Code{1}){
			case 'A':
				$text.='Ace';
				break;
			case '2':
				$text.='Two';
				break;
			case '3':
				$text.='Three';
				break;
			case '4':
				$text.='Four';
				break;
			case '5':
				$text.='Five';
				break;
			case '6':
				$text.='Six';
				break;
			case '7':
				$text.='Seven';
				break;
			case '8':
				$text.='Eight';
				break;
			case '9':
				$text.='Nine';
				break;
			case 'J':
				$text.='Jack';
				break;
			case 'Q':
				$text.='Queen';
				break;
			case 'K':
				$text.='King';
				break;
			default:
				return false;
		}
		return $text;
	}
}
?>
